"""Auto-generated model profiles.

DO NOT EDIT THIS FILE MANUALLY.
This file is generated by the langchain-profiles CLI tool.

It contains data derived from the models.dev project.

Source: https://github.com/sst/models.dev
License: MIT License

To update these data, refer to the instructions here:

https://docs.langchain.com/oss/python/langchain/models#updating-or-overwriting-profile-data
"""

from typing import Any

_PROFILES: dict[str, dict[str, Any]] = {
    "claude-3-5-haiku-20241022": {
        "max_input_tokens": 200000,
        "max_output_tokens": 8192,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-3-5-haiku-latest": {
        "max_input_tokens": 200000,
        "max_output_tokens": 8192,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-3-5-sonnet-20240620": {
        "max_input_tokens": 200000,
        "max_output_tokens": 8192,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-3-5-sonnet-20241022": {
        "max_input_tokens": 200000,
        "max_output_tokens": 8192,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-3-7-sonnet-20250219": {
        "max_input_tokens": 200000,
        "max_output_tokens": 64000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-3-7-sonnet-latest": {
        "max_input_tokens": 200000,
        "max_output_tokens": 64000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-3-haiku-20240307": {
        "max_input_tokens": 200000,
        "max_output_tokens": 4096,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-3-opus-20240229": {
        "max_input_tokens": 200000,
        "max_output_tokens": 4096,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-3-sonnet-20240229": {
        "max_input_tokens": 200000,
        "max_output_tokens": 4096,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-haiku-4-5": {
        "max_input_tokens": 200000,
        "max_output_tokens": 64000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-haiku-4-5-20251001": {
        "max_input_tokens": 200000,
        "max_output_tokens": 64000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-opus-4-0": {
        "max_input_tokens": 200000,
        "max_output_tokens": 32000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-opus-4-1": {
        "max_input_tokens": 200000,
        "max_output_tokens": 32000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": True,
    },
    "claude-opus-4-1-20250805": {
        "max_input_tokens": 200000,
        "max_output_tokens": 32000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-opus-4-20250514": {
        "max_input_tokens": 200000,
        "max_output_tokens": 32000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-opus-4-5": {
        "max_input_tokens": 200000,
        "max_output_tokens": 64000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-opus-4-5-20251101": {
        "max_input_tokens": 200000,
        "max_output_tokens": 64000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-opus-4-6": {
        "max_input_tokens": 200000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-sonnet-4-0": {
        "max_input_tokens": 200000,
        "max_output_tokens": 64000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-sonnet-4-20250514": {
        "max_input_tokens": 200000,
        "max_output_tokens": 64000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-sonnet-4-5": {
        "max_input_tokens": 200000,
        "max_output_tokens": 64000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": True,
    },
    "claude-sonnet-4-5-20250929": {
        "max_input_tokens": 200000,
        "max_output_tokens": 64000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
    "claude-sonnet-4-6": {
        "max_input_tokens": 200000,
        "max_output_tokens": 64000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "structured_output": False,
    },
}
